<?php
/*
 * copyright (c) 2010 MDBitz - Matthew John Denton - mdbitz.com
 *
 * This file is part of WordPress Amazon Associate Plugin.
 *
 * WordPress Amazon Associate is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * WordPress Amazon Associate is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WordPress Amazon Associate.
 * If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * WPAA_ShortCodeHandler
 *
 * This file contains the class WPAA_ShortCodeHandler
 *
 * @author Matthew John Denton <matt@mdbitz.com>
 * @package com.mdbitz.wordpress.amazon_associate
 */

/**
 * Short Code Handler Class for WordPress Amazon Associate plugin
 *
 * @package com.mdbitz.wordpress.amazon_associate
 */
class WPAA_ShortCodeHandler {

    /**
     * APaPi library instance
     * @var AmazonProductAPI
     */
    protected $apapi = null;

    /**
     * Constructor
     */
    function __construct( $mappings = null) {
        add_shortcode('amazon_link', array(&$this,'amazonLinkHandler'));
        add_shortcode('amazon_image', array(&$this,'amazonImageHandler'));
        add_shortcode('amazon_carousel', array(&$this,'amazonCarouselHandler'));
        add_shortcode('amazon_mp3_clips', array(&$this,'amazonMP3ClipsHandler'));
        add_shortcode('amazon_my_favorites', array(&$this,'amazonMyFavoritesHandler'));
        add_shortcode('amazon_search', array(&$this,'amazonSearchHandler'));
        if( $mappings != null ) {
            foreach( $mappings as $mapping) {
                foreach( $mapping as $name => $function ) {
                    add_shortcode( $name, array(&$this,$function));
                }
            }
        }
    }

    /**
     * set the APaPi library instance
     * @param AmazonProductAPI $apapi
     */
    public function setAPI( $apapi ) {
        $this->apapi = $apapi;
    }

    /**
     * Amazon Link Short Code Handler 
     * @param array $atts
     * @param string $content
     * @param string $code
     */
    public function amazonLinkHandler( $atts, $content=null, $code="" ) {
        global $wpaa;
        extract( shortcode_atts( array( 'id'=>null, 'locale'=> null, 'type' => 'ASIN', 'target' => '_blank' ), $atts ) );
        $output_str = $content;
        if( ! is_null( $id ) ) {
            $locale = $wpaa->getGeoLocale( $locale );
            $this->apapi->setLocale( $locale );
            $this->apapi->setAssociateId( $wpaa->getAssociateId( $locale ) );
            $result = $this->apapi->lookup( $id, $type );
            if( $result->isSuccess() && isset( $result->Items[0] ) ) {
                $output_str = '<a href="' . urldecode($result->Items[0]->DetailPageURL) . '" target="' . $target . '">' . $content . '</a>';
            }
        }
        return $output_str;
    }

    /**
     * Amazon Image Short Code Handler
     * @param array $atts
     * @param string $content
     * @param string $code
     */
    public function amazonImageHandler( $atts, $content=null, $code="" ) {
        global $wpaa;
        extract( shortcode_atts( array( 'id'=>null, 'locale'=>null, 'size'=>'medium', 'type' => 'ASIN', 'link' => false, 'target' => '_blank' ), $atts ) );
        $output_str = $content;
        if( ! is_null( $id ) ) {
            $locale = $wpaa->getGeoLocale( $locale );
            $this->apapi->setLocale( $locale );
            $this->apapi->setAssociateId( $wpaa->getAssociateId( $locale ) );
            $result = $this->apapi->lookup( $id, $type, array( AmazonProduct_ResponseGroup::ITEM_ATTRIBUTES, AmazonProduct_ResponseGroup::IMAGES ) );
            if( $result->isSuccess() && isset( $result->Items[0] ) ) {
                $size = strtolower($size );
                $image = null;
                switch( $size ) {
                    case "small":
                        $image = $result->Items[0]->SmallImage;
                        break;
                    case "large":
                        $image = $result->Items[0]->LargeImage;
                        break;
                    default :
                        $image = $result->Items[0]->MediumImage;
                        break;
                }
                $output_str = '';
                if( $link ) {
                    $output_str = '<a href="' . urldecode($result->Items[0]->DetailPageURL) . '" target="' . $target . '">';
                }
                if( ! is_null( $image ) ) {
                    $output_str .= $image->toHTML();
                } else {
                    $output_str .= $content;
                }
                if( $link ) {
                    $output_str .= '</a>';
                }
            }
        }
        return $output_str;
    }

    /**
     * Amazon Carousel Short Code Handler
     * @param array $atts
     * @param string $content
     * @param string $code
     */
    public function amazonCarouselHandler( $atts, $content=null, $code="" ) {
        $carousel = new AmazonWidget_Carousel( $this->mergeWidgetOptions( AmazonWidget_Carousel::getDefaultShortCodeOptions(), $atts ) );
        return $carousel->toHTML();
    }

    /**
     * Amazon MP3 Clips Short Code Handler
     * @param array $atts
     * @param string $content
     * @param string $code
     */
    public function amazonMP3ClipsHandler( $atts, $content=null, $code="" ) {
        $carousel = new AmazonWidget_MP3Clips( $this->mergeWidgetOptions( AmazonWidget_MP3Clips::getDefaultShortCodeOptions(), $atts ) );
        return $carousel->toHTML();
    }

    /**
     * Amazon My Favorites Short Code Handler
     * @param array $atts
     * @param string $content
     * @param string $code
     */
    public function amazonMyFavoritesHandler( $atts, $content=null, $code="" ) {
        $carousel = new AmazonWidget_MyFavorites( $this->mergeWidgetOptions( AmazonWidget_MyFavorites::getDefaultShortCodeOptions(), $atts ) );
        return $carousel->toHTML();
    }

    /**
     * Amazon Search Short Code Handler
     * @param array $atts
     * @param string $content
     * @param string $code
     */
    public function amazonSearchHandler( $atts, $content=null, $code="" ) {
        $carousel = new AmazonWidget_Search( $this->mergeWidgetOptions( AmazonWidget_Search::getDefaultShortCodeOptions(), $atts ) );
        return $carousel->toHTML();
    }

    /**
     * Merge Widget Options and include plugin compliance settings
     * @param Array $defaultArgs
     * @param Array $atts
     * @return Array
     */
    private function mergeWidgetOptions( $defaultArgs, $atts ) {
        if( isset( $atts['marketplace'] ) ) {
            $atts['market_place'] = $atts['marketplace'];
            unset( $atts['marketplace'] );
        }
        return shortcode_atts( $defaultArgs, $atts );
    }

}